/******************************************************************************
*                                                                             *
*    DTUtil.cpp                             Copyright(c) 2003-2009 itow,y.    *
*                                                                             *
******************************************************************************/

/******************************************************************************

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

******************************************************************************/


#include <stdlib.h>
#include <windows.h>
#include "hspdll.h"




EXPORT BOOL WINAPI MultiplyDivide(void *pResult,int p2,int p3,int p4)
{
	*(int*)pResult=MulDiv(p2,p3,p4);
	return 0;
}




#define DIFF_TIME(start,end) \
	((end)>=(start)?(end)-(start):0xFFFFFFFFUL-(start)+1+(end))


static DWORD dwStartTime,dwStopTime;
static BOOL fStopped=FALSE;


EXPORT BOOL WINAPI StartTimeCount(int,int,int,int)
{
	dwStartTime=timeGetTime();
	fStopped=FALSE;
	return 0;
}


EXPORT BOOL WINAPI GetTimeCount(void *pVal,int,int,int)
{
	DWORD dwTime;

	if (!fStopped)
		dwTime=timeGetTime();
	else
		dwTime=dwStopTime;
	*(int*)pVal=DIFF_TIME(dwStartTime,dwTime)/10;
	return 0;
}


EXPORT BOOL WINAPI StopTimeCount(int,int,int,int)
{
	if (!fStopped) {
		dwStopTime=timeGetTime();
		fStopped=TRUE;
	}
	return 0;
}


EXPORT BOOL WINAPI RestartTimeCount(int,int,int,int)
{
	if (fStopped) {
		DWORD dwCurTime=GetTickCount();

		dwStartTime+=DIFF_TIME(dwStopTime,dwCurTime);
		fStopped=FALSE;
	}
	return 0;
}




static BMSCR *pSrcBmscr=NULL;


EXPORT BOOL WINAPI SetPixelizeSrc(BMSCR *pBmscr,int,int,int)
{
	pSrcBmscr=pBmscr;
	return 0;
}


EXPORT BOOL WINAPI Pixelize(BMSCR *pDstBmscr,int BoxWidth,int BoxHeight,int)
{
	int Rows,Columns,x,y,x1,y1,sx,sy,i;
	int SrcLineBytes,DstLineBytes,DstLineDWord;
	unsigned int b,g,r;
	BYTE *p;
	DWORD *pdwSrcBit,*pdwDstBit;

	if (pSrcBmscr==NULL)
		return -1;
	if (pSrcBmscr->palmode || pDstBmscr->palmode)
		return -2;
	if (BoxWidth<=0 || BoxHeight<=0)
		return -3;
	Rows=(pDstBmscr->sx>=pSrcBmscr->sx)?pSrcBmscr->sx:pDstBmscr->sx;
	Columns=(pDstBmscr->sy>=pSrcBmscr->sy)?pSrcBmscr->sy:pDstBmscr->sy;
	SrcLineBytes=(pSrcBmscr->sx*3+3)/4*4;
	DstLineDWord=(pDstBmscr->sx*3+3)/4;
	DstLineBytes=DstLineDWord*4;
	for (y=0;y<Columns;y+=BoxHeight) {
		sy=(y+BoxHeight<=Columns)?BoxHeight:Columns-y;
		for (x=0;x<Rows;x+=BoxWidth) {
			b=g=r=0;
			sx=(x+BoxWidth<=Rows)?BoxWidth:Rows-x;
			for (y1=0;y1<sy;y1++) {
				p=pSrcBmscr->pBit+SrcLineBytes*(pSrcBmscr->sy-1-(y+y1))+x*3;
				for (x1=0;x1<sx;x1++) {
					b+=*p++;
					g+=*p++;
					r+=*p++;
				}
			}
			b/=sx*sy;
			g/=sx*sy;
			r/=sx*sy;
			p=pDstBmscr->pBit+DstLineBytes*(pDstBmscr->sy-1-y)+x*3;
			for (x1=0;x1<sx;x1++) {
				*p++=b;
				*p++=g;
				*p++=r;
			}
		}
		for (y1=1;y1<sy;y1++) {
			pdwDstBit=(DWORD*)(pDstBmscr->pBit+
										DstLineBytes*(pDstBmscr->sy-1-(y+y1)));
			pdwSrcBit=(DWORD*)(pDstBmscr->pBit+
										DstLineBytes*(pDstBmscr->sy-(y+y1)));
			for (i=0;i<DstLineDWord;i++) {
				*pdwDstBit++=*pdwSrcBit++;
			}
		}
	}
	return 0;
}




static BMSCR *pFadeBmscr=NULL;
static int FadeOpacity;


EXPORT BOOL WINAPI SetFadeMode(BMSCR *pBmscr,int op,int,int)
{
	pFadeBmscr=pBmscr;
	FadeOpacity=op<0?0:op>256?256:op;
	return 0;
}


EXPORT BOOL WINAPI Fade(int Left,int Top,int Width,int Height)
{
	int sx,sy,x,y;
	int LineBytes;
	int r,g,b;
	BYTE *p;

	if (pFadeBmscr==NULL)
		return -1;
	if (pFadeBmscr->palmode)
		return -2;
	if (Width<=0 || Height<=0 || Left>=pFadeBmscr->sx || Top>=pFadeBmscr->sy)
		return -3;
	if (FadeOpacity==0)
		return 0;
	r=GetRValue(pFadeBmscr->color);
	g=GetGValue(pFadeBmscr->color);
	b=GetBValue(pFadeBmscr->color);
	sx=(pFadeBmscr->sx>=Left+Width)?Width:pFadeBmscr->sx-Left;
	sy=(pFadeBmscr->sy>=Top+Height)?Height:pFadeBmscr->sy-Top;
	LineBytes=(pFadeBmscr->sx*3+3)/4*4;
	for (y=Top;y<Top+sy;y++) {
		p=pFadeBmscr->pBit+LineBytes*(pFadeBmscr->sy-1-y)+Left*3;
		for (x=Left;x<Left+sx;x++) {
			*p++=((*p*(256-FadeOpacity))+(b*FadeOpacity))>>8;
			*p++=((*p*(256-FadeOpacity))+(g*FadeOpacity))>>8;
			*p++=((*p*(256-FadeOpacity))+(r*FadeOpacity))>>8;
		}
	}
	return 0;
}




static LPSTR MakeFilter(LPCSTR pszString)
{
	LPCSTR p;
	LPSTR pszFilter,q;

	pszFilter=(LPSTR)malloc(lstrlen(pszString)+2);
	if (pszFilter==NULL)
		return NULL;
	p=pszString;
	q=pszFilter;
	while (*p!='\0') {
		if (*p=='|') {
			*q++='\0';
		} else {
			if (IsDBCSLeadByteEx(CP_ACP,*p))
				*q++=*p++;
			*q++=*p;
		}
		p++;
	}
	*q++='\0';
	*q='\0';
	return pszFilter;
}


EXPORT BOOL WINAPI FileOpenDialog(HSPEXINFO *phei,int,int,int)
{
	LPSTR pszFilter;
	int *pnFilterIndex;
	char szFileName[MAX_PATH],szTitle[64];;
	unsigned int uFlags;
	OPENFILENAME ofn;

	pszFilter=MakeFilter((*phei->HspFunc_prm_gets)());
	if (pszFilter==NULL)
		return 1;
	pnFilterIndex=(int*)(*phei->HspFunc_prm_getv)();
	lstrcpyn(szFileName,(*phei->HspFunc_prm_getds)(""),MAX_PATH);
	lstrcpyn(szTitle,(*phei->HspFunc_prm_getds)(""),64);
	uFlags=(unsigned int)(*phei->HspFunc_prm_getdi)(0);
	if (*phei->er!=0) {
		free(pszFilter);
		return *phei->er;
	}
#if _WIN32_WINNT>=0x0500
	ofn.lStructSize=OPENFILENAME_SIZE_VERSION_400;
#else
	ofn.lStructSize=sizeof(OPENFILENAME);
#endif
	ofn.hwndOwner=((BMSCR*)(*phei->HspFunc_getbmscr)(*phei->actscr))->hwnd;
	ofn.lpstrFilter=pszFilter;
	ofn.lpstrCustomFilter=NULL;
	ofn.nFilterIndex=*pnFilterIndex;
	ofn.lpstrFile=szFileName;
	ofn.nMaxFile=MAX_PATH;
	ofn.lpstrFileTitle=NULL;
	ofn.lpstrInitialDir=NULL;
	ofn.lpstrTitle=szTitle[0]!='\0'?szTitle:NULL;
	ofn.Flags=uFlags;
	ofn.lpstrDefExt=NULL;
	if (!GetOpenFileName(&ofn)) {
		free(pszFilter);
		return 0;
	}
	free(pszFilter);
	lstrcpy(phei->refstr,szFileName);
	*pnFilterIndex=ofn.nFilterIndex;
	return -1;
}
